import { useEffect, useState } from 'react';
import dayjs from "dayjs";
import { useTranslation } from 'react-i18next';
import i18n from '@/i18n';

export const useDatePresets = () => {
  const [presets, setPresets] = useState([]);
  const { t, i18n } = useTranslation();

  useEffect(() => {
    const newPresets = [
      {
        label: t("datetime.thisWeek"),
        value: [dayjs().startOf("w"), dayjs().endOf("w")],
      },
      {
        label: t("datetime.lastWeek"),
        value: [dayjs().startOf("w").subtract(7, "days"), dayjs().endOf("w").subtract(7, "days")],
      },
      {
        label: t("datetime.thisMonth"),
        value: [dayjs().startOf("M"), dayjs().endOf("M")],
      },
      {
        label: t("datetime.lastMonth"),
        value: [dayjs().subtract(1, "M").startOf("M"), dayjs().subtract(1, "M").endOf("M")],
      },
      {
        label: t("datetime.nextMonth"),
        value: [dayjs().add(1, "M").startOf("M"), dayjs().add(1, "M").endOf("M")],
      },
      {
        label: t("datetime.lastThreeMonth"),
        value: [dayjs().subtract(2, "M").startOf("M"), dayjs().endOf("M")],
      },
      {
        label: t("datetime.nextThreeMonth"),
        value: [dayjs().startOf("M"), dayjs().add(3,"M").endOf("M")],
      },
      {
        label: t("datetime.firstHalfYear"),
        value: [dayjs().startOf("y"), dayjs().endOf("y").subtract(6, "M")],
      },
      {
        label: t("datetime.latterHalfYear"),
        value: [dayjs().startOf("y").add(6,"M"), dayjs().endOf("y")],
      },
      {
        label: t("datetime.thisYear"),
        value: [dayjs().startOf("y"), dayjs().endOf("y")],
      },
    ];
    setPresets(newPresets);
  }, [i18n.language]);

  return presets;
}

export const useWeekdays = () => {
  const [data, setData] = useState([]);
  const { t, i18n } = useTranslation();
  useEffect(() => {
    const newData = [
      { value: '1', label: t('weekdays.1') },
      { value: '2', label: t('weekdays.2') },
      { value: '3', label: t('weekdays.3') },
      { value: '4', label: t('weekdays.4') },
      { value: '5', label: t('weekdays.5') },
      { value: '6', label: t('weekdays.6') },
      { value: '7', label: t('weekdays.7') },
    ];
    setData(newData);
    return () => {};
  }, [i18n.language]);
  return data;
};
