import { create } from "zustand";
import { fetchJSON, postForm } from "@/utils/request";
import { prepareUrl, isNotEmpty } from "@/utils/commons";
import { HT_HOST, DATE_FORMAT } from "@/config";
import dayjs from "dayjs";

const airTicketStore = create((set, get) => ({
	loading: false,
	setLoading: loading => set({ loading }),
	setPlanList: planList => set({ planList }),
	setPlanDetail: planDetail => set({ planDetail }),
	setGuestList: guestList => set({ guestList }),
	setVEIFlightBill: vEIFlightBill => set({ vEIFlightBill }),
	setVeiPlanChangeTxt: veiPlanChangeTxt => set({ veiPlanChangeTxt }),
	setAirPortList: airPortList => set({ airPortList }),

	async getPlanList(vei_sn, GRI_Name, TimeStart, TimeEnd, plan_state, airticket_state) {
		const { setLoading, setPlanList } = get();
		setLoading(true);
		const searchParams = {
			vei_sn: vei_sn,
			FlightDate1: TimeStart,
			FlightDate2: TimeEnd,
			GRI_Name: GRI_Name,
			FlightStatus: plan_state,
			TicketIssued: airticket_state,
		};

		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/GetFlightPlan`, searchParams);
		const _result = errcode !== 0 ? [] : result;
		setPlanList(_result);
		setLoading(false);
	},

	async getPlanDetail(vei_sn, gri_sn) {
		const { setPlanDetail } = get();
		const searchParams = {
			vei_sn: vei_sn,
			gri_sn: gri_sn,
		};
		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/GetFlightPlanDetail`, searchParams);
		const _result = errcode !== 0 ? [] : result;
		setPlanDetail(_result);
		//return _result.filter(item => isNotEmpty(item.GRI_No));
		return "dsadsd";
	},

	async postFlightDetail(CLF_SN, GRI_SN, VEI_SN, original_values, info_object) {
		const formData = new FormData();
		formData.append("CLF_SN", CLF_SN ? CLF_SN : "");
		formData.append("GRI_SN", GRI_SN);
		formData.append("VEI_SN", VEI_SN);
		for (const [key, value] of Object.entries(original_values)) {
			formData.append(key, value); //先用原始数据填充一遍，确保复制了全部数据到新表
		}
		for (const [key, value] of Object.entries(info_object)) {
			formData.set(key, value); //再用新值覆盖
		}
		formData.set("StartDate", dayjs(info_object.StartDate).format(DATE_FORMAT)); //再用新值覆盖
		//是否出票的值，true、false变为1或0
		formData.set("TicketIssued", info_object.TicketIssued ? 1 : 0);
		const postUrl = HT_HOST + "/Service_BaseInfoWeb/edit_or_new_flight_info";
		return postForm(postUrl, formData).then(json => {
			if (json.errcode == 0) {
				return json;
			} else {
				throw new Error(json.errmsg + ": " + json.errcode);
			}
		});
	},
	//删除航班信息
	async delete_flight_info(CLF_SN) {
		const searchParams = {
			CLF_SN: CLF_SN,
		};
		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/Delete_flight_info`, searchParams);
		const _result = errcode !== 0 ? [] : result;
		return _result;
	},

	async getGuestList(coli_sn) {
		const { setGuestList } = get();
		const searchParams = {
			COLI_SN: coli_sn,
		};
		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/GetFlightGuestInfo`, searchParams);
		const _result = errcode !== 0 ? [] : result;
		setGuestList(_result);
	},
	//获取账单列表
	async getVEIFlightBill(VEI_SN, GRI_Name, CheckStatus, FlightDate1, FlightDate2) {
		const { setLoading, setVEIFlightBill } = get();
		setLoading(true);
		const searchParams = {
			VEI_SN: VEI_SN,
			GRI_Name: GRI_Name,
			CheckStatus: CheckStatus,
			FlightDate1: FlightDate1,
			FlightDate2: FlightDate2,
		};
		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/GetVEIFlightBill`, searchParams);
		const _result = errcode !== 0 ? [] : result;
		setVEIFlightBill(_result);
		setLoading(false);
	},
	//保存费用
	async postFlightCost(values) {
		const formData = new FormData();
		for (const [key, value] of Object.entries(values)) {
			formData.append(key, value); //先用原始数据填充一遍，确保复制了全部数据到新表
		}
		const postUrl = HT_HOST + "/Service_BaseInfoWeb/edit_or_new_flight_cost";
		return postForm(postUrl, formData).then(json => {
			if (json.errcode == 0) {
				return json;
			} else {
				throw new Error(json.errmsg + ": " + json.errcode);
			}
		});
	},
	//删除费用
	async deleteFlightCost(CLC_SN) {
		const searchParams = {
			CLC_SN: CLC_SN,
		};
		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/Delete_flight_cost`, searchParams);
		const _result = errcode !== 0 ? [] : result;
		return _result;
	},
	//获取变更信息
	async getVeiPlanChange(VEI_SN, GRI_SN) {
		const { setVeiPlanChangeTxt } = get();
		const searchParams = {
			VEI_SN: VEI_SN,
			GRI_SN: GRI_SN,
		};
		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/GetVeiFlightPlanChange`, searchParams);
		const _result = errcode !== 0 ? [] : result;
		setVeiPlanChangeTxt(_result);
	},
	//提交变更确认
	async postVeiFlightPlanConfirm(VEI_SN, GRI_SN, LMI_SN, ConfirmInfo) {
		const formData = new FormData();
		formData.append("VEI_SN", VEI_SN);
		formData.append("GRI_SN", GRI_SN);
		formData.append("LMI_SN", LMI_SN);
		formData.append("ConfirmInfo", ConfirmInfo);
		const postUrl = HT_HOST + "/Service_BaseInfoWeb/VeiFlightPlanConfirm";
		return postForm(postUrl, formData).then(json => {
			if (json.errcode == 0) {
				return json;
			} else {
				throw new Error(json.errmsg + ": " + json.errcode);
			}
		});
	},
	//提交账单
	async postVEIFlightBillSubmit(VEI_SN, values) {
		const formData = new FormData();
		formData.append("vei_sn", VEI_SN);
		formData.append("billdata", JSON.stringify(values));
		const postUrl = HT_HOST + "/Service_BaseInfoWeb/VEIFlightBillSubmit";
		return postForm(postUrl, formData).then(json => {
			if (json.errcode == 0) {
				return json;
			} else {
				throw new Error(json.errmsg + ": " + json.errcode);
			}
		});
	},
	//通知顾问查看机票信息
	async ticketIssuedNotifications(LMI_SN, CLF_SN, OPI_SN, FlightMemo_messages) {
		const searchParams = {
			CLF_SN: CLF_SN,
			OPI_SN: OPI_SN,
			LMI_SN: LMI_SN,
			FlightMemo_messages: FlightMemo_messages,
		};
		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/TicketIssuedNotifications`, searchParams);
		const _result = errcode !== 0 ? [] : result;
		return _result;
	},
	//获取机场列表
	async getAirPortList() {
		const { setAirPortList } = get();
		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/GetAirPortInfo`);
		const _result = errcode !== 0 ? [] : result;
		setAirPortList(_result);
	},

	airLineList: [
		{ label: "CA-国航", value: "国航" },
		{ label: "MU-东方航空", value: "东方航空" },
		{ label: "FM-上海航空", value: "上海航空" },
		{ label: "CZ-南方航空", value: "南方航空" },
		{ label: "HO-吉祥航空", value: "吉祥航空" },
		{ label: "HU-海南航空", value: "海南航空" },
		{ label: "ZH-深圳航空", value: "深圳航空" },
		{ label: "MF-厦门航空", value: "厦门航空" },
		{ label: "3U-四川航空", value: "四川航空" },
		{ label: "SC-山东航空", value: "山东航空" },
		{ label: "JD-首都航空", value: "首都航空" },
		{ label: "BK-奥凯航空", value: "奥凯航空" },
		{ label: "GS-天津航空", value: "天津航空" },
		{ label: "CN-大新华", value: "大新华" },
		{ label: "KN-中联航", value: "中联航" },
		{ label: "TV-西藏航空", value: "西藏航空" },
		{ label: "8L-祥鹏航空", value: "祥鹏航空" },
		{ label: "KY-昆明航空", value: "昆明航空" },
		{ label: "EU-成都航空", value: "成都航空" },
		{ label: "G5-华夏航空", value: "华夏航空" },
		{ label: "NS-河北航空", value: "河北航空" },
		{ label: "QW-青岛航空", value: "青岛航空" },
		{ label: "Y8-扬子江", value: "扬子江" },
		{ label: "PN-西部航空", value: "西部航空" },
		{ label: "DZ-东海航空", value: "东海航空" },
		{ label: "GT-桂林航空", value: "桂林航空" },
		{ label: "9H-长安航空", value: "长安航空" },
		{ label: "GY-多彩航空", value: "多彩航空" },
		{ label: "DR-瑞丽航空", value: "瑞丽航空" },
		{ label: "GJ-长龙航空", value: "长龙航空" },
		{ label: "GX-广西北部", value: "广西北部" },
	],
}));

export default airTicketStore;
