import { create } from "zustand";
import { fetchJSON, postForm } from "@/utils/request";
import { prepareUrl, isNotEmpty } from "@/utils/commons";
import { HT_HOST, DATE_FORMAT } from "@/config";
import dayjs from "dayjs";

const trainTicketStore = create((set, get) => ({
	loading: false,
	setLoading: loading => set({ loading }),
	setPlanList: planList => set({ planList }),
	setPlanDetail: planDetail => set({ planDetail }),
	setGuestList: guestList => set({ guestList }),
	setVEIFlightBill: vEIFlightBill => set({ vEIFlightBill }),
	setVeiPlanChangeTxt: veiPlanChangeTxt => set({ veiPlanChangeTxt }),

	async getPlanList(vei_sn, GRI_Name, TimeStart, TimeEnd, plan_state, airticket_state) {
		const { setLoading, setPlanList } = get();
		setLoading(true);
		const searchParams = {
			vei_sn: vei_sn,
			FlightDate1: TimeStart,
			FlightDate2: TimeEnd,
			GRI_Name: GRI_Name,
			FlightStatus: plan_state,
			TicketIssued: airticket_state,
		};

		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/GetTrainPlan`, searchParams);
		const _result = errcode !== 0 ? [] : result;
		setPlanList(_result);
		setLoading(false);
	},

	async getPlanDetail(vei_sn, gri_sn) {
		const { setPlanDetail } = get();
		const searchParams = {
			vei_sn: vei_sn,
			gri_sn: gri_sn,
		};
		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/GetTrainPlanDetail`, searchParams);
		const _result = errcode !== 0 ? [] : result;
		setPlanDetail(_result);
		//return _result.filter(item => isNotEmpty(item.GRI_No));
		return "dsadsd";
	},

	async postFlightDetail(CLF_SN, GRI_SN, VEI_SN, original_values, info_object) {
		const formData = new FormData();
		formData.append("CLF_SN", CLF_SN ? CLF_SN : "");
		formData.append("GRI_SN", GRI_SN);
		formData.append("VEI_SN", VEI_SN);
		for (const [key, value] of Object.entries(original_values)) {
			formData.append(key, value); //先用原始数据填充一遍，确保复制了全部数据到新表
		}
		for (const [key, value] of Object.entries(info_object)) {
			formData.set(key, value); //再用新值覆盖
		}
		formData.set("StartDate", dayjs(info_object.StartDate).format(DATE_FORMAT)); //再用新值覆盖
		//是否出票的值，true、false变为1或0
		formData.set("TicketIssued", info_object.TicketIssued ? 1 : 0);
		const postUrl = HT_HOST + "/Service_BaseInfoWeb/edit_or_new_flight_info";
		return postForm(postUrl, formData).then(json => {
			if (json.errcode == 0) {
				return json;
			} else {
				throw new Error(json.errmsg + ": " + json.errcode);
			}
		});
	},
	//删除火车信息
	async delete_flight_info(CLF_SN) {
		const searchParams = {
			CLF_SN: CLF_SN,
		};
		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/Delete_flight_info`, searchParams);
		const _result = errcode !== 0 ? [] : result;
		return _result;
	},

	async getGuestList(coli_sn) {
		const { setGuestList } = get();
		const searchParams = {
			COLI_SN: coli_sn,
		};
		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/GetFlightGuestInfo`, searchParams);
		const _result = errcode !== 0 ? [] : result;
		setGuestList(_result);
	},
	//获取账单列表
	async getVEIFlightBill(VEI_SN, GRI_Name, CheckStatus, FlightDate1, FlightDate2) {
		const { setLoading, setVEIFlightBill } = get();
		setLoading(true);
		const searchParams = {
			VEI_SN: VEI_SN,
			GRI_Name: GRI_Name,
			CheckStatus: CheckStatus,
			FlightDate1: FlightDate1,
			FlightDate2: FlightDate2,
			ServiceType: 2,
		};
		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/GetVEIFlightBill`, searchParams);
		const _result = errcode !== 0 ? [] : result;
		setVEIFlightBill(_result);
		setLoading(false);
	},
	//保存费用
	async postFlightCost(values) {
		const formData = new FormData();
		for (const [key, value] of Object.entries(values)) {
			formData.append(key, value); //先用原始数据填充一遍，确保复制了全部数据到新表
		}
		const postUrl = HT_HOST + "/Service_BaseInfoWeb/edit_or_new_flight_cost";
		return postForm(postUrl, formData).then(json => {
			if (json.errcode == 0) {
				return json;
			} else {
				throw new Error(json.errmsg + ": " + json.errcode);
			}
		});
	},
	//删除费用
	async deleteFlightCost(CLC_SN) {
		const searchParams = {
			CLC_SN: CLC_SN,
		};
		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/Delete_flight_cost`, searchParams);
		const _result = errcode !== 0 ? [] : result;
		return _result;
	},
	//获取变更信息
	async getVeiPlanChange(VEI_SN, GRI_SN) {
		const { setVeiPlanChangeTxt } = get();
		const searchParams = {
			VEI_SN: VEI_SN,
			GRI_SN: GRI_SN,
		};
		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/GetVeiFlightPlanChange`, searchParams);
		const _result = errcode !== 0 ? [] : result;
		setVeiPlanChangeTxt(_result);
	},
	//提交变更确认
	async postVeiFlightPlanConfirm(VEI_SN, GRI_SN, LMI_SN, ConfirmInfo) {
		const formData = new FormData();
		formData.append("VEI_SN", VEI_SN);
		formData.append("GRI_SN", GRI_SN);
		formData.append("LMI_SN", LMI_SN);
		formData.append("ServiceType", 2);
		formData.append("ConfirmInfo", ConfirmInfo);
		const postUrl = HT_HOST + "/Service_BaseInfoWeb/VeiFlightPlanConfirm";
		return postForm(postUrl, formData).then(json => {
			if (json.errcode == 0) {
				return json;
			} else {
				throw new Error(json.errmsg + ": " + json.errcode);
			}
		});
	},
	//提交账单
	async postVEIFlightBillSubmit(VEI_SN, values) {
		const formData = new FormData();
		formData.append("vei_sn", VEI_SN);
		formData.append("ServiceType", 2);
		formData.append("billdata", JSON.stringify(values));
		const postUrl = HT_HOST + "/Service_BaseInfoWeb/VEIFlightBillSubmit";
		return postForm(postUrl, formData).then(json => {
			if (json.errcode == 0) {
				return json;
			} else {
				throw new Error(json.errmsg + ": " + json.errcode);
			}
		});
	},
	//通知顾问查看车票信息
	async ticketIssuedNotifications(LMI_SN, CLF_SN, OPI_SN, FlightMemo_messages) {
		const searchParams = {
			CLF_SN: CLF_SN,
			OPI_SN: OPI_SN,
			LMI_SN: LMI_SN,
			ServiceType: 2,
			FlightMemo_messages: FlightMemo_messages,
		};
		const { errcode, result } = await fetchJSON(`${HT_HOST}/Service_BaseInfoWeb/TicketIssuedNotifications`, searchParams);
		const _result = errcode !== 0 ? [] : result;
		return _result;
	},


	// 定义座位表对象
	seatTable: [
		{ code: "9", name: "商务座", enName: "Business Class Seat" },
		{ code: "P", name: "特等座", enName: "Superior Seat" },
		{ code: "M", name: "一等座", enName: "First Class Seat" },
		{ code: "O", name: "二等座", enName: "Second Class Seat" },
		{ code: "6", name: "高级软卧", enName: "Superior Soft Sleeper" },
		{ code: "5", name: "包厢硬卧", enName: "Hard Sleeper in Private Compartment" },
		{ code: "4", name: "软卧", enName: "Soft Sleeper" },
		{ code: "F", name: "动卧", enName: "High - Speed Train Sleeper" },
		{ code: "3", name: "硬卧", enName: "Hard Sleeper" },
		{ code: "2", name: "软座", enName: "Soft Seat" },
		{ code: "1", name: "硬座", enName: "Hard Seat" },
		{ code: "Q", name: "观光座", enName: "Sightseeing Seat" },
		{ code: "H", name: "其它", enName: "Other" },
		{ code: "WZ", name: "无座", enName: "Standing - room Only" },
		{ code: "YDW", name: "一等卧", enName: "First - Class Sleeper" },
		{ code: "EDW", name: "二等卧", enName: "Second - Class Sleeper" },
	],
}));

export default trainTicketStore;
