import { useNavigate } from "react-router-dom"
import { Button, Space, Form, Input, Row, Typography, App } from 'antd'
import { useTranslation } from 'react-i18next'
import useAuthStore from '@/stores/Auth'

const { Title } = Typography

function ChangePassword() {

  const { t } = useTranslation()
  const navigate = useNavigate()
  const changeUserPassword = useAuthStore((state) => state.changeUserPassword)
  const { notification } = App.useApp()
  const [form] = Form.useForm()

  const onFinish = (values) => {
    changeUserPassword(values.currentPassword, values.newPassword)
      .then(() => {
        notification.success({
          message: t('Validation.Title'),
          description: t('account:Validation.Success'),
          placement: 'top',
          duration: 4,
        })
        form.resetFields()
      })
      .catch(() => {
        notification.error({
          message: t('Validation.Title'),
          description: t('account:Validation.Fail'),
          placement: 'top',
          duration: 4,
        })
      })
  }

  const onFinishFailed = (errorInfo) => {
    console.log('Failed:', errorInfo)
    // form.resetFields()
  }

  return (
    <>
    <Row justify="center" align="middle" className='min-h-96'>
      <Form
        name="basic"
        form={form}
        layout="vertical"
        size="large"
        className='max-w-xl'
        onFinish={onFinish}
        onFinishFailed={onFinishFailed}
        autoComplete="off"
      >
        <Form.Item><Title level={2}>{t('ChangePassword')}</Title></Form.Item>
        <Form.Item
          label={t('account:CurrentPassword')}
          name="currentPassword"
          rules={[
            {
              required: true,
              message: t('account:Validation.CurrentPassword'),
            },
          ]}
        >
          <Input.Password />
        </Form.Item>
        <Form.Item
          label={t('account:NewPassword')}
          name="newPassword"
          rules={[
            {
              required: true,
              message: t('account:Validation.NewPassword'),
            },
          ]}
        >
          <Input.Password />
        </Form.Item>
        <Form.Item
          label={t('account:ReenterPassword')}
          name="reenterPassword"
          rules={[
            {
              required: true,
              message: t('account:Validation.ReenterPassword'),
            },
          ]}
        >
          <Input.Password />
        </Form.Item>
        <Form.Item>
        <Space size="middle">
          <Button type="primary" htmlType="submit">
            {t('Submit')}
          </Button>
          <Button onClick={() => navigate('/reservation/newest')}>
          {t('Cancel')}
          </Button>
        </Space>
        </Form.Item>
      </Form>
    </Row>
    </>
  )
}

export default ChangePassword