import { useState, useEffect } from "react";
import { Grid, Divider, Layout, Steps, Statistic, Col, Row, Space, Checkbox, Table, Button, App, Typography } from "antd";
import { PhoneOutlined, CustomerServiceOutlined, FrownTwoTone, LikeTwoTone } from "@ant-design/icons";
import { useParams, useHref, useNavigate, NavLink } from "react-router-dom";
import { isEmpty, formatColonTime } from "@/utils/commons";
import dayjs from "dayjs";
import SearchForm from "@/components/SearchForm";
import BackBtn from "@/components/BackBtn";
import { TableExportBtn } from "@/components/Data";

import airTicketStore from "@/stores/Airticket";
import { usingStorage } from "@/hooks/usingStorage";

const Invoice = props => {
	const navigate = useNavigate();
	const { notification } = App.useApp();
	const { travelAgencyId } = usingStorage();
	const [getVEIFlightBill, vEIFlightBill, loading, postVEIFlightBillSubmit] = airTicketStore(state => [state.getVEIFlightBill, state.vEIFlightBill, state.loading, state.postVEIFlightBillSubmit]);
	const showTotal = total => `合计 ${total} `;
	const [selectedValues, setSelectedValues] = useState([]);

	const vEIFlightBillColumns = [
		{
			title: "团名",
			key: "GRI_No",
			dataIndex: "GRI_No",
			render: (text, record) => (
				<Typography.Text title={record.Memo}>
					{record.GRI_No} {record.WL}
				</Typography.Text>
			),
		},
		{
			title: "状态",
			key: "CostType",
			dataIndex: "CostType",
		},
		{
			title: "出发日期",
			key: "StartDate",
			dataIndex: "StartDate",
			sorter: (a, b) => {
				const dateA = new Date(a.StartDate);
				const dateB = new Date(b.StartDate);
				return dateB.getTime() - dateA.getTime();
			},
		},
		{
			title: "出发",
			key: "FromCity",
			dataIndex: "FromCity",
			render: (text, record) => (record.CostType == "出票" ? `${record.FromCity}` : "-"),
		},
		{
			title: "抵达",
			key: "ToCity",
			dataIndex: "ToCity",
			render: (text, record) => (record.CostType == "出票" ? `${record.ToCity}` : "-"),
		},
		{
			title: "航班",
			key: "FlightNo",
			dataIndex: "FlightNo",
			render: (text, record) => (record.CostType == "出票" ? text : "-"),
		},
		{
			title: "PNR",
			key: "PNR",
			dataIndex: "PNR",
			render: (text, record) => (record.CostType == "出票" ? text : "-"),
		},
		{
			title: "票号",
			key: "TicketNo",
			dataIndex: "TicketNo",
			render: (text, record) => (record.CostType == "出票" ? text : "-"),
		},
		{
			title: "机票类型",
			key: "FlightType",
			dataIndex: "FlightType",
			render: (text, record) => (record.CostType == "出票" ? text : "-"),
		},
		{
			title: "机票价格",
			children: [
				{
					title: vEIFlightBill && vEIFlightBill.reduce((acc, curr) => acc + curr.Cost, 0),
					dataIndex: "Cost",
					render: (text, record) => (record.CostType == "出票" ? text : "-"),
				},
			],
			key: "Cost",
		},
		{
			title: "服务费",
			children: [
				{
					title: vEIFlightBill && vEIFlightBill.reduce((acc, curr) => acc + curr.ServiceFee, 0),
					dataIndex: "ServiceFee",
				},
			],
		},
		{
			title: "折扣",
			key: "Discount",
			dataIndex: "Discount",
			render: (text, record) => (record.CostType == "出票" ? text : "-"),
		},
		{
			title: "审核状态",
			children: [
				{
					title: (
						<Button type="link" onClick={() => checkALL()}>
							全选
						</Button>
					),
					dataIndex: "CheckStatus", //状态小于2，表示还未提交审核
					render: (text, record) =>
						record.CheckStatus < 2 ? (
							<Checkbox onChange={event => handleCheckboxChange(event, record)} checked={checkboxStates(record.CLC_SN)}>
								待提交
							</Checkbox>
						) : (
							<Steps
								size="small"
								current={record.CheckStatus - 1}
								items={[
									{
										title: "提交账单",
									},
									{
										title: "顾问审核",
									},
									{
										title: "财务处理",
									},
									{
										title: "账单支付",
									},
								]}
							/>
						),
				},
			],

			sorter: (a, b) => {
				return b.CheckStatus - a.CheckStatus;
			},
		},
	];
	//获取当前状态
	const checkboxStates = CLC_SN => {
		return selectedValues.some(v => v.CLC_SN === CLC_SN);
	};
	// 处理checkbox改变事件
	const handleCheckboxChange = (event, data) => {
		const value = { CLC_SN: data.CLC_SN, WL: data.WL, OPI_SN: data.OPI_SN, OPI_Email: data.OPI_Email, GRI_SN: data.GRI_SN, GRI_Name: data.GRI_Name, Cost: data.Cost, ServiceFee: data.ServiceFee };
		if (event.target.checked) {
			setSelectedValues([...selectedValues, value]); // 如果被选中，则添加到数组
		} else {
			setSelectedValues(selectedValues.filter(v => v.CLC_SN !== value.CLC_SN)); // 如果取消选中，则从数组中移除
		}
	};
	//提交账单
	const postInvoice = () => {
		postVEIFlightBillSubmit(travelAgencyId, selectedValues)
			.then(() => {
				notification.success({
					message: `成功`,
					description: "账单提交成功!",
					placement: "top",
					duration: 4,
					icon: <LikeTwoTone />,
				});
				setSelectedValues([]); //清空选中的项目
			})
			.catch(() => {
				notification.error({
					message: `错误`,
					description: "保存失败",
					placement: "top",
					duration: 4,
					icon: <FrownTwoTone />,
				});
			});
	};
	//全选或不全选
	const checkALL = () => {
		if (isEmpty(vEIFlightBill)) return;
		const allChecked = selectedValues.length === vEIFlightBill.filter(item => item.CheckStatus < 2).length;
		setSelectedValues(
			allChecked
				? []
				: vEIFlightBill
						.filter(item => item.CheckStatus < 2)
						.map(item => ({ CLC_SN: item.CLC_SN, WL: item.WL, OPI_SN: item.OPI_SN, OPI_Email: item.OPI_Email, GRI_SN: item.GRI_SN, GRI_Name: item.GRI_Name, Cost: item.Cost, ServiceFee: item.ServiceFee }))
		); // 如果全选，则清空数组，否则添加所有项
	};

	useEffect(() => {}, []);

	return (
		<Space direction="vertical" style={{ width: "100%" }}>
			<Row>
				<Col md={20} lg={20} xxl={20}>
					<SearchForm
						initialValue={{
							dates: [dayjs().startOf("M"), dayjs().endOf("M")],
						}}
						fieldsConfig={{
							shows: ["referenceNo", "dates", "invoiceCheckStatus"],
							fieldProps: {
								referenceNo: { label: "搜索计划" },
								dates: { label: "出发日期", col: 8 },
							},
						}}
						onSubmit={(err, formVal, filedsVal) => {
							getVEIFlightBill(travelAgencyId, formVal.referenceNo, formVal.invoiceCheckStatus, formVal.startdate, formVal.endtime);
						}}
					/>
				</Col>
				<Col md={4} lg={4} xxl={4}>
					<BackBtn to={"/airticket"} />
				</Col>
			</Row>

			<Row>
				<Col md={24} lg={24} xxl={24}>
					<Table bordered={true} rowKey="CLC_SN" columns={vEIFlightBillColumns} dataSource={vEIFlightBill} loading={loading} pagination={{ defaultPageSize: 100, showTotal: showTotal }} />

					<TableExportBtn btnTxt="导出账单" label={`机票账单`} {...{ columns: vEIFlightBillColumns, dataSource: vEIFlightBill }} />
				</Col>

				<Col md={24} lg={24} xxl={24}></Col>
			</Row>
			<Row>
				<Col md={24} lg={18} xxl={18}></Col>
				<Col md={24} lg={2} xxl={2}>
					<Statistic title="已选机票价格" value={selectedValues.reduce((acc, curr) => acc + curr.Cost, 0)} />
				</Col>
				<Col md={24} lg={2} xxl={2}>
					<Statistic title="已选服务费" value={selectedValues.reduce((acc, curr) => acc + curr.ServiceFee, 0)} />
				</Col>
				<Col md={24} lg={2} xxl={2}>
					<Button type="primary" size="large" onClick={postInvoice} disabled={selectedValues.length == 0}>
						提交账单
					</Button>
				</Col>
			</Row>
		</Space>
	);
};
export default Invoice;
