import { useParams, useNavigate } from "react-router-dom";
import { useEffect, useState } from "react";
import { Row, Col, Space, Button, Typography, Card, Form, Upload, Input, Divider, DatePicker, Select, App, Descriptions, Image } from "antd";
import { PlusOutlined,AuditOutlined } from "@ant-design/icons";
import { isNotEmpty } from "@/utils/commons";
import * as config from "@/config";
import dayjs from "dayjs";
import { fetchInvoiceDetail, postEditInvoiceDetail, postAddInvoice } from "@/stores/Invoice";
import { removeFeedbackImages } from "@/stores/Feedback";
import BackBtn from "@/components/BackBtn";
import { usingStorage } from "@/hooks/usingStorage";

const { Title, Text } = Typography;

function Detail() {
	const navigate = useNavigate();
	const { GMDSN, GSN } = useParams();
	const { userId, travelAgencyId, loginToken } = usingStorage();
	const [form] = Form.useForm();
	const [dataLoading, setDataLoading] = useState(false);
	const [edited, setEdited] = useState(true); //表单是否允许编辑
	const { formCurrency, onCurrencyChange } = useState();
	const { notification } = App.useApp();

	const [invoicePicList, setInvoicePicList] = useState([]);

	const [invoiceZDDetail, setInvoiceZDDetail] = useState([]);
	const [invoiceCurrencyList, setInvoiceCurrencyList] = useState([]);
	const [invoiceProductList, setInvoiceProductList] = useState([]);
	const [invoiceGroupInfo, setInvoiceGroupInfo] = useState({});

	const [invoiceFormData, setInvoiceFormData] = useState({});
	const [invoicekImages, setInvoicekImages] = useState([]);

	useEffect(() => {
		console.info("Detail.useEffect: " + GMDSN + "/" + GSN);
		defaultShow();
	}, [GMDSN, GSN]);

	function defaultShow() {
		setDataLoading(true);

		fetchInvoiceDetail(travelAgencyId, GMDSN, GSN)
			.then(json => {
				setInvoiceZDDetail(json.invoiceZDDetail);
				setInvoiceGroupInfo(json.invoiceGroupInfo);
				setInvoiceProductList(json.invoiceProductList);
				setInvoiceCurrencyList(json.invoiceCurrencyList);

				let ZDDetail = json.invoiceZDDetail;
				if (isNotEmpty(ZDDetail)) {
					let arrLen = ZDDetail.length;
					const formData = ZDDetail.map((data, index) => {
						if (data.GMD_Dealed == false && arrLen == index + 1) {
							//只有最后一条账单未审核通过才显示
							const _formData = { info_money: data.GMD_Cost, info_Currency: data.GMD_Currency, info_date: isNotEmpty(data.GMD_PayDate) ? dayjs(data.GMD_PayDate) : "", info_gmdsn: data.GMD_SN };
							setInvoiceFormData(_formData);
							return _formData;
						}
					});

					if (form) {
						form.setFieldsValue(formData[arrLen - 1]); //{'info_money':'111','info_Currency':'THB','info_date':''}
					}

					//图片列表
					let arrPicList = ZDDetail.map((data, index) => {
						const GMD_Pic = data.GMD_Pic;
						let picList = [];
						if (isNotEmpty(GMD_Pic)) {
							let js_Pic = JSON.parse(GMD_Pic);
							picList = js_Pic.map((picData, pic_Index) => {
								return {
									uid: -pic_Index, //用负数，防止添加删除的时候错误
									name: "",
									status: "done",
									url: picData.url,
								};
							});
						}
						if (data.GMD_Dealed == false && arrLen == index + 1) {
							setInvoicekImages(picList);
						}
						return picList;
					});
					setInvoicePicList(arrPicList);
				}
			})
			.catch(ex => {
				notification.error({
					message: `Notification`,
					description: ex.message,
					placement: "top",
					duration: 4,
				});
			})
			.finally(() => {
				setDataLoading(false);
			});
	}

	const fileList = invoicekImages;
	//图片列表
	let arrimg = [];
	if (isNotEmpty(fileList)) {
		arrimg = fileList.map((data, index) => {
			return {
				url: data.url,
			};
		});
	}
	const onFinish = values => {
		const fieldVaule = {
			...values,
			info_date: isNotEmpty(values["info_date"]) ? values["info_date"].format("YYYY-MM") : null,
			info_images: JSON.stringify(arrimg),
		};
		// console.log("Success:", fieldVaule);
		//入库
		if (fieldVaule) {
			postEditInvoiceDetail(userId, fieldVaule.info_gmdsn, fieldVaule.info_Currency, fieldVaule.info_money, fieldVaule.info_date, fieldVaule.info_images, "").then(data => {
				// console.log(data);
				let param = { info_money: fieldVaule.info_money, info_Currency: fieldVaule.info_Currency, info_date: fieldVaule.info_date };
				setInvoiceFormData(param);
				if (data.errcode == 0) {
					setEdited(true);
					notification.success({
						message: `Notification`,
						description: "Success Submit!",
						placement: "top",
						duration: 4,
					});
				}
			});
		}
	};

	const handleChange = info => {
		console.log(info);
		let newFileList = [...info.fileList];
		newFileList = newFileList.map(file => {
			if (file.response && file.response.result) {
				file.url = file.response.result.file_url;
			}
			return file;
		});
		setInvoicekImages(newFileList);
	};

	const handRemove = info => {
		console.log(info);
		removeFeedbackImages(info.url);
		return true;
	};

	//币种
	function bindCurrency() {
		let arr = [];
		arr = invoiceCurrencyList.map((data, index) => {
			return {
				value: data.CRI_Code,
				label: data.CRI_Name,
			};
		});

		return arr;
	}

	function addInvoice() {
		postAddInvoice(userId, travelAgencyId, GSN, "", 0, "", "[]", "")
			.then(data => {})
			.finally(() => {
				defaultShow();
			});
	}

	function addButton(check) {
		if (check) {
			return (
				<Row>
					<Divider orientation="left"></Divider>
					<Button
						type="primary"
						block
						onClick={() => {
							addInvoice(confirm);
							setEdited(false);
						}}>
						ADD New Invoice
					</Button>
				</Row>
			);
		}
	}

	const invoiceStatus = FKState => {
		switch (FKState - 1) {
			case 1:
				return "Submitted";
			case 2:
				return "Travel Advisor";
			case 3:
				return "Finance Dept";
			case 4:
				return "Paid";
			default:
				return "";
		}
	};

	//循环生成多次报账信息
	function bindSubmitForm() {
		if (isNotEmpty(invoiceZDDetail)) {
			let submitForm = invoiceZDDetail.map((data, index) => {
				if (data.GMD_Dealed) {
					//已审核的直接显示信息即可，无需表单
					return (
						<Row key={data.GMD_SN} gutter={16}>
							<Col span={4}></Col>
							<Col span={16}>
								<Card type="inner" title={"Invoice " + (index + 1)}>
									<Row gutter={16}>
										<Col span={12}>
											<Descriptions column={1}>
												<Descriptions.Item label="Amount">{data.GMD_Cost}</Descriptions.Item>
												<Descriptions.Item label="Currency">{data.GMD_Currency}</Descriptions.Item>
												<Descriptions.Item label="Due Date">{data.GMD_PayDate}</Descriptions.Item>
												<Descriptions.Item label="Status">{invoiceStatus(data.FKState)}</Descriptions.Item>
											</Descriptions>
										</Col>
										<Col span={12}>
											<Image.PreviewGroup>
												{invoicePicList[index] &&
													invoicePicList[index].map(item => {
														if (item.url) {
															return <Image key={item.uid} width={90} src={item.url} />;
														}
													})}
											</Image.PreviewGroup>
										</Col>
									</Row>
								</Card>
								{addButton(index + 1 == invoiceZDDetail.length)}
							</Col>
							<Col span={4}></Col>
						</Row>
					);
				} else {
					// 一个团只能有一个未审核的账单记录
					return (
						<Row key={data.GMD_SN} gutter={16}>
							<Col span={4}></Col>
							<Col span={16}>
								<Card
									type="inner"
									title={"Invoice " + (index + 1)}
									extra={
										<Button type="link" onClick={() => setEdited(false)}>
											Edit
										</Button>
									}>
									<Form name="invoice_submit" onFinish={onFinish} form={form} disabled={edited}>
										<Row gutter={16}>
											<Col span={12}>
												<Form.Item
													name="info_money"
													label="Amount"
													rules={[
														{
															required: true,
															message: "Please input your money!",
														},
													]}>
													<Input />
												</Form.Item>

												<Form.Item
													name="info_Currency"
													label="Currency"
													rules={[
														{
															required: true,
															message: "Please select Currency type!",
														},
													]}>
													<Select placeholder="Select Currency type" onChange={onCurrencyChange} options={bindCurrency()}></Select>
												</Form.Item>
												<Form.Item
													name="info_date"
													label="Due Month"
													rules={[
														{
															required: true,
															message: "please select Due Month!",
														},
													]}>
													<DatePicker picker="month" />
												</Form.Item>
												<Text type="secondary">
													Payment is arranged during the last week of each month. If the invoice is issued after the 20th, please select the following month for payment. For urgent payments, please contact the
													travel advisor.{" "}
												</Text>
												<Form.Item name="info_gmdsn" hidden={true}>
													<input />
												</Form.Item>
												<br />
												<Form.Item>
													<Button type="primary" htmlType="submit">
														Submit
													</Button>
												</Form.Item>
											</Col>
											<Col span={12}>
												<Form.Item>
													<Upload
														name="ghhfile"
														multiple={true}
														action={config.OVERSEA_HOST + `/service-fileServer/FileUpload?GRI_SN=${GSN}&VEI_SN=${travelAgencyId}&FilePathName=invoice&token=${loginToken}`}
														fileList={fileList}
														listType="picture-card"
														onChange={handleChange}
														onRemove={handRemove}
														accept=".jpg,.png,.peg,.bmp">
														<div>
															<PlusOutlined />
															<div style={{ marginTop: 8 }}>Upload Invoice</div>
														</div>
													</Upload>
												</Form.Item>
											</Col>
										</Row>
									</Form>
								</Card>
							</Col>
							<Col span={4}></Col>
						</Row>
					);
				}
			});
			return submitForm;
		} else {
			return (
				<Row gutter={16} style={{ backgroundColor: "#f6f7f9", width: "100%", padding: "20px 40px" }}>
					<Col span={4}></Col>
					<Col span={18}>{addButton(1 == 1)}</Col>
					<Col span={4}></Col>
				</Row>
			);
		}
	}

	return (
		<>
			<Space direction="vertical" style={{ width: "100%" }}>
				<Row gutter={16}>
					<Col span={20}>
						<Title level={4}>Reference Number: {invoiceGroupInfo.VGroupInfo}</Title>
					</Col>
					<Col span={4}>
						<BackBtn />
						<Button icon={<AuditOutlined />} onClick={() => navigate(`/invoice/history/0/338787`)}>
							Billing Records
						</Button>
					</Col>
				</Row>
				<Title level={5}></Title>
				{bindSubmitForm()}
			</Space>
		</>
	);
}

export default Detail;
