import { NavLink, useNavigate } from "react-router-dom";
import { Row, Col, Space, Button, Table, App, Steps } from "antd";
import { formatDate, isNotEmpty } from "@/utils/commons";
import { AuditOutlined, SmileOutlined, SolutionOutlined, EditOutlined } from "@ant-design/icons";
import SearchForm from '@/components/SearchForm';
import dayjs from 'dayjs';
import useInvoiceStore from '@/stores/Invoice';
import { usingStorage } from "@/hooks/usingStorage";


function Index() {
  const {travelAgencyId, } = usingStorage();
  const [invoiceList, fetchInvoiceList] = useInvoiceStore((state) => [state.invoiceList, state.fetchInvoiceList]);
	const navigate = useNavigate();
	const { notification } = App.useApp();
	const showTotal = total => `Total ${total} items`;

	const invoiceListColumns = [
		{
			title: "Ref.No",
			dataIndex: "GroupName",
			key: "GroupName",
			render: (text, record) => <NavLink to={`/invoice/detail/${record.key}/${record.gmd_gri_sn}`}>{text}</NavLink>,
		},
		{
			title: "Arrival Date",
			key: "GetGDate",
			dataIndex: "GetGDate",
			render: (text, record) => (isNotEmpty(text) ? formatDate(new Date(text)) : ""),
		},
		{
			title: "Total Amount",
			key: "AllMoney",
			dataIndex: "AllMoney",
			render: (text, record) => (isNotEmpty(record.GMD_Currency) ? record.GMD_Currency + " " + text : text),
		},
		{
			title: "Status",
			key: "status",
			render: BillStatus,
		},
	];

	function BillStatus(text, record) {
		let FKState = record.FKState - 1;
		return (
			<Steps
				current={FKState}
				initial={1}
				items={[
					{
						title: "Submitted",
						icon: <EditOutlined />,
					},
					{
						title: "Travel Advisor",
						icon: <SolutionOutlined />,
					},
					{
						title: "Finance Dept",
						icon: <AuditOutlined />,
					},
					{
						title: "Paid",
						icon: <SmileOutlined />,
					},
				]}
			/>
		);
	}

	return (
		<Space direction="vertical" style={{ width: "100%" }}>
			<Row gutter={16}>
				<Col md={16} sm={16} xs={24}  >
					<SearchForm
					initialValue={{
					dates: [dayjs().subtract(2, 'M').startOf('M'), dayjs().endOf('M')],
					}}
					fieldsConfig={{
					shows: ['referenceNo', 'invoiceStatus', 'dates'],
					fieldProps: {
						referenceNo: { col: 7 },
						invoiceStatus: { col: 4},
						dates: { col: 10 },
					},
					}}
					onSubmit={(err, formVal, filedsVal) => {
					fetchInvoiceList(travelAgencyId, formVal.referenceNo, formVal.startdate, formVal.enddate, formVal.invoiceStatus);
					}}
					/>
				</Col>
				<Col md={8} sm={8} xs={24} >
					<Space>
						<Button icon={<AuditOutlined />} onClick={() => navigate(`/invoice/detail/0/395074`)}>
							Misc. Invoice
						</Button>
						<Button icon={<AuditOutlined />} onClick={() => navigate(`/invoice/paid`)}>
							Bank statement
						</Button>
					</Space>
				</Col>
			</Row>
			<Row>
				<Col md={24} lg={24} xxl={24}>
					<Table bordered pagination={{ defaultPageSize: 20, showTotal: showTotal }} columns={invoiceListColumns} dataSource={(invoiceList)} />
				</Col>
			</Row>
		</Space>
	);
}

export default (Index);
//value={[search_date_start, search_date_end]} 20230803 zp 去掉日期默认可以清空
