import { NavLink, useNavigate } from 'react-router-dom';
import { useEffect } from 'react';
import { Row, Col, Space, Table, Image } from 'antd';
import { formatDate, isNotEmpty } from '@/utils/commons';
import SearchForm from '@/components/SearchForm';
import dayjs from 'dayjs';
import BackBtn from '@/components/BackBtn';
import useInvoiceStore from '@/stores/Invoice';
import { usingStorage } from '@/hooks/usingStorage';

function Paid() {
  const {travelAgencyId, } = usingStorage();
  const [invoicePaid, fetchInvoicePaid] = useInvoiceStore((state) => [state.invoicePaid, state.fetchInvoicePaid]);
  const navigate = useNavigate();
  const showTotal = (total) => `Total ${total} items`;

  useEffect(() => {
    fetchInvoicePaid(travelAgencyId, '', '', '');
  }, []);
  const invoicePaidColumns = [
    {
      title: 'Payment ref.NO',
      dataIndex: 'fl_finaceNo',
      key: 'fl_finaceNo',
      render: (text, record) => <NavLink to={`/invoice/paid/detail/${record.key}`}>{text}</NavLink>,
    },
    {
      title: 'Payment date',
      key: 'fl_adddate',
      dataIndex: 'fl_adddate',
      render: (text, record) => (isNotEmpty(text) ? formatDate(new Date(text)) : ''),
    },
    {
      title: 'Number of bills',
      key: 'fcount',
      dataIndex: 'fcount',
    },
    {
      title: 'Total amount',
      key: 'pSum',
      dataIndex: 'pSum',
      //render: (text, record) => (isNotEmpty(record.GMD_Currency) ? record.GMD_Currency + " " + text : text),
    },
    {
      title: 'Bank statement',
      key: 'fl_pic',
      dataIndex: 'fl_pic',
      render: showPIc,
    },
  ];

  function showPIc(text, record) {
    let strPic = record.fl_pic;
    //console.log(JSON.parse(strPic));
    if (isNotEmpty(strPic)) {
      return JSON.parse(strPic).map((item, index) => {
        return <Image key={index} width={90} src={item.url} />;
      });
    } else {
      return '';
    }
  }

  return (
    <Space direction='vertical' style={{ width: '100%' }}>
      <Row gutter={16}>
        <Col span={20}>
          <SearchForm
            initialValue={{
              dates: [dayjs().subtract(2, 'M').startOf('M'), dayjs().endOf('M')],
            }}
            fieldsConfig={{
              shows: ['referenceNo', 'dates'],
              fieldProps: {
                referenceNo: { col: 5 },
                dates: { col: 10, label: 'Date' },
              },
            }}
            onSubmit={(err, formVal, filedsVal) => {
              fetchInvoicePaid(travelAgencyId, formVal.referenceNo, formVal.startdate, formVal.enddate);
            }}
          />
        </Col>
        <Col span={4}>
          <BackBtn to={'/invoice'} />
        </Col>
      </Row>
      <Row>
        <Col md={24} lg={24} xxl={24}>
          <Table bordered columns={invoicePaidColumns} dataSource={(invoicePaid)} />
        </Col>
      </Row>
    </Space>
  );
}

export default (Paid);
