import { useEffect, useState } from 'react';
import { App, Breadcrumb, Divider } from 'antd';
import { useTranslation } from 'react-i18next';
import { useProductsTypesMapVal, useNewProductRecord } from '@/hooks/useProductsSets';
import useProductsStore, { postProductsSaveAction } from '@/stores/Products/Index';
import useAuthStore from '@/stores/Auth';
import { PERM_PRODUCTS_MANAGEMENT, PERM_PRODUCTS_OFFER_PUT, PERM_PRODUCTS_INFO_PUT, PERM_PRODUCTS_NEW } from '@/config';
import { isEmpty, pick } from '@/utils/commons';
import ProductInfoForm from './ProductInfoForm';
import { usingStorage } from '@/hooks/usingStorage';
import Extras from './Extras';
import NewProductModal from './NewProductModal';

const ProductInfo = ({ ...props }) => {
  const { t } = useTranslation();
  const { notification, message } = App.useApp();
  const { userId } = usingStorage();
  const isPermitted = useAuthStore((state) => state.isPermitted);
  const productsTypesMapVal = useProductsTypesMapVal();
  const newProductRecord = useNewProductRecord();

  const [loading, setLoading, appendNewProduct] = useProductsStore((state) => [state.loading, state.setLoading, state.appendNewProduct]);
  const [activeAgency, editingProduct, setEditingProduct] = useProductsStore((state) => [state.activeAgency, state.editingProduct, state.setEditingProduct]);

  const [extrasVisible, setExtrasVisible] = useState(false);
  const [editablePerm, setEditablePerm] = useState(false);
  const [infoEditable, setInfoEditable] = useState(false);
  const [priceEditable, setPriceEditable] = useState(false);
  useEffect(() => {
    const topPerm = isPermitted(PERM_PRODUCTS_MANAGEMENT); // 高级权限

    const hasHT = (editingProduct?.info?.htid || 0) > 0;
    // const hasAuditPer = isPermitted(PERM_PRODUCTS_OFFER_AUDIT);
    const hasEditPer = isPermitted(PERM_PRODUCTS_INFO_PUT) || isPermitted(PERM_PRODUCTS_NEW); // || isPermitted(PERM_PRODUCTS_OFFER_PUT);

    setEditablePerm(topPerm || hasEditPer);
    // setEditable(topPerm || (hasAuditPer ? true : (!hasHT && hasEditPer)));
    // setEditable(true); // debug: 0
    // console.log('editable', hasAuditPer, (notAudit && hasEditPer));
    setInfoEditable(topPerm || (!hasHT && hasEditPer));

    const _priceEditable = [-1, 3].includes(activeAgency?.audit_state_id) || isEmpty(editingProduct?.info?.id);
    const hasPricePer = isPermitted(PERM_PRODUCTS_OFFER_PUT);
    // setPriceEditable(topPerm || (_priceEditable && hasPricePer));
    setPriceEditable(topPerm || (hasPricePer));
    // setPriceEditable(true); // debug: 0

    const showExtras = topPerm && hasHT; // !isEmpty(editingProduct) &&
    setExtrasVisible(showExtras);

    setLgcEdits({});
    setInfoEditStatus('');
    return () => {};
  }, [activeAgency, editingProduct]);

  const [infoEditStatus, setInfoEditStatus] = useState('');
  const [lgcEdits, setLgcEdits] = useState({});
  const onValuesChange = (changedValues, forms) => {
    // console.log('onValuesChange', changedValues);
    if ('product_title' in changedValues) {
      setInfoEditStatus('2');
      setLgcEdits({...lgcEdits, '2': {'edit_status': '2'}});
    }
    if ('lgc_details_mapped' in changedValues) {
      const lgc = Object.keys(changedValues.lgc_details_mapped)[0];
      setLgcEdits({...lgcEdits, [lgc]: {'edit_status': '2'}});
    } else {
      setInfoEditStatus('2');
    }
  };

  const onSave = async (err, values, forms) => {
    values.travel_agency_id = activeAgency.travel_agency_id;
    const copyNewProduct = structuredClone(newProductRecord);
    const poster = {
      // ...(topPerm ? { } : { 'audit_state': -1 }), // 高级权限: 不变更状态值
      // "create_date": dayjs().format('YYYY-MM-DD HH:mm:ss'),
      // "created_by": userId,
      'travel_agency_id': activeAgency.travel_agency_id,
      // "travel_agency_name": "",
      // "lastedit_changed": "",
      "edit_status": infoEditStatus || editingProduct.info.edit_status,
    };
    const copyFields = pick(editingProduct.info, ['product_type_id']); // 'title',
    const readyToSubInfo = { ...copyNewProduct.info, ...editingProduct.info, title: editingProduct.info.product_title, ...values.info, ...copyFields, ...poster };
    // console.log('onSave', editingProduct.info, readyToSubInfo);

    /** lgc_details */
    const prevLgcDetailsMapped = editingProduct.lgc_details.reduce((r, c) => ({ ...r, [c.lgc]: c }), {});
    const mergedLgc = { ...prevLgcDetailsMapped, ...values.lgc_details_mapped, };
    for (const lgcKey in lgcEdits) {
      if (Object.prototype.hasOwnProperty.call(lgcEdits, lgcKey)) {
        const element = lgcEdits[lgcKey];
        mergedLgc[lgcKey].edit_status = element?.edit_status || values.lgc_details_mapped[lgcKey]?.edit_status || '2';
      }
    }

    // console.log('before save', '\n lgcEdits:', lgcEdits, '\n mergedLgc', mergedLgc);
    // return false; // debug: 0
    /** 提交保存 */
    setLoading(true);
    const { success, result } = await postProductsSaveAction({
      travel_agency_id: activeAgency.travel_agency_id,
      info: readyToSubInfo,
      lgc_details: Object.values(mergedLgc),
      quotation: values.quotation.map((q) => ({ ...q, unit: Number(q.unit || q.unit_id), unit_id: Number(q.unit_id) })), // || editingProduct.quotation, // 没改动, 就用原来的
    }).catch((ex) => {
      setLoading(false);
      notification.error({
        message: 'Notification',
        description: ex.message,
        placement: 'top',
        duration: 4,
      });
    });
    setLoading(false);
    success ? message.success(t('Success')) : message.error(t('Failed'));
    // 保存后更新数据
    // result.quotation = isEmpty(result.quotation) ? editingProduct.quotation : result.quotation;
    result.info.htid = editingProduct?.info?.htid;
    appendNewProduct(result);
    setEditingProduct(result);
  };
  return (
    <>
      <Breadcrumb
        items={[
          { title: productsTypesMapVal[editingProduct?.info?.product_type_id]?.label || editingProduct?.info?.product_type_name },
          { title: editingProduct?.info?.title ?? t('New') },
          // { title: 'htID: ' + editingProduct?.info?.htid },
          // { title: 'ID: ' + editingProduct?.info?.id },
        ]}
      />
      <Divider className='my-1' />
      {isEmpty(editingProduct) ? (
        <div className=' my-2'>
          <NewProductModal />
        </div>
      ) : (
        <>
          <h2>{t('products:EditComponents.info')}</h2>
          <ProductInfoForm {...{ editablePerm, infoEditable, priceEditable, onValuesChange }} initialValues={editingProduct?.info} onSubmit={onSave} />
          <Divider className='my-1' />
          {extrasVisible && <Extras productId={editingProduct?.info?.id} />}
        </>
      )}
    </>
  );
};
export default ProductInfo;
