import { useEffect, useState } from 'react';
import { App, Form, Input, Row, Col, Select, Button, InputNumber, Checkbox } from 'antd';
import { objectMapper, isEmpty, isNotEmpty } from '@/utils/commons';
import { useTranslation } from 'react-i18next';
import { useWeekdays } from '@/hooks/useDatePresets';
import DeptSelector from '@/components/DeptSelector';
import CitySelector from '@/components/CitySelector';
import { useProductsTypesFieldsets } from '@/hooks/useProductsSets';
import useProductsStore from '@/stores/Products/Index';
import ProductInfoLgc from './ProductInfoLgc';
import ProductInfoQuotation from './ProductInfoQuotation';
import { useHTLanguageSetsMapVal } from '@/hooks/useHTLanguageSets';

const InfoForm = ({ onSubmit, onReset, onValuesChange, editablePerm, infoEditable, priceEditable, showSubmit, confirmText, formName, ...props }) => {
  const { notification } = App.useApp();
  const { t } = useTranslation('products');
  const HTLanguageSetsMapVal = useHTLanguageSetsMapVal();
  const [loading, editingProduct] = useProductsStore((state) => [state.loading, state.editingProduct]);
  const weekdays = useWeekdays();
  const [form] = Form.useForm();
  const { sort, hides, fieldProps, fieldComProps } = {
    sort: '',
    fieldProps: '',
    fieldComProps: '',
    hides: [],
    shows: [],
    ...props.fieldsConfig,
  };
  const filedsets = useProductsTypesFieldsets(editingProduct?.info?.product_type_id);
  const shows = filedsets[0];

  const [pickEditedInfo, setPickEditedInfo] = useState({}); // 传递联动的字段

  // const [editable, setEditable] = useState(true);
  const [formEditable, setFormEditable] = useState(true);
  const [showSave, setShowSave] = useState(true);
  useEffect(() => {
    form.resetFields();
    form.setFieldValue('city', editingProduct?.info?.city_id ? { value: editingProduct?.info?.city_id, label: editingProduct?.info?.city_name } : undefined);
    form.setFieldValue('dept', { value: editingProduct?.info?.dept_id, label: editingProduct?.info?.dept_name });
    const lgc_details_mapped = (editingProduct?.lgc_details || []).reduce((r, c) => ({ ...r, [c.lgc]: c }), {});
    form.setFieldValue('lgc_details_mapped', lgc_details_mapped);
    form.setFieldValue('quotation', editingProduct?.quotation);
    form.setFieldValue('display_to_c', editingProduct.info?.display_to_c || '0');
    setPickEditedInfo({ ...pickEditedInfo, product_title: editingProduct?.info?.product_title });

    setFormEditable(infoEditable || priceEditable);

    // const editable0 = isEmpty(editingProduct) ? false : editablePerm; // 空对象未操作
    setShowSave(infoEditable || priceEditable);
    // setEditable(editable0);
    return () => {};
  }, [editingProduct, editablePerm, infoEditable, priceEditable]);

  const onFinish = (values) => {
    console.log('Received values of form, origin form value: \n', values);
    const dest = formValuesMapper(values);
    console.log('form value send to onSubmit:\n', dest);
    if (typeof onSubmit === 'function') {
      onSubmit(null, dest, values);
    }
  };

  const onFinishFailed = ({ values, errorFields }) => {
    console.log('form validate failed', '\nform values:', values, '\nerrorFields', errorFields);
    notification.warning({
      message: '数据未填写完整',
      // description: '数据未填写完整',
      placement: 'top',
      duration: 4,
    })
  };

  const handleReset = () => {
    form.setFieldsValue({
      // 'DateType': undefined,
    });
    if (typeof onReset === 'function') {
      onReset();
    }
  };
  const onIValuesChange = (changedValues, allValues) => {
    const dest = formValuesMapper(allValues);
    // console.log('form onValuesChange', Object.keys(changedValues), changedValues);
    if ('product_title' in changedValues) {
      const editTitle = (changedValues.product_title);
      setPickEditedInfo({ ...pickEditedInfo, product_title: editTitle });
    }
    if (typeof onValuesChange === 'function') {
      onValuesChange(changedValues, dest);
    }
  };
  const onFieldsChange = (hangedFields, allFields) => {
    console.log('onFieldsChange', hangedFields, allFields);
  };
  return (
    <>
      <Form
        form={form}
        disabled={!formEditable}
        name={formName || 'product_info'}
        // preserve={false}
        onFinish={onFinish}
        onValuesChange={onIValuesChange}
        // onFieldsChange={onFieldsChange}
        initialValues={editingProduct?.info}
        onFinishFailed={onFinishFailed} scrollToFirstError >
        <Row>
          {getFields({ sort, initialValue: editingProduct?.info, hides, shows, fieldProps, fieldComProps, form, t, dataSets: { weekdays }, editable: infoEditable })}
          {/* {showSubmit && (
            <Col flex='1 0 90px' className='flex justify-end items-start'>
              <Space align='center'>
                <Button size={'middle'} type='primary' htmlType='submit' loading={loading}>
                  {confirmText || t('common:Save')}
                </Button>
              </Space>
            </Col>
          )} */}
        </Row>
        {/* <Divider className='my-1' /> */}
        <Form.Item className='mb-0'
          name={'lgc_details_mapped'}
          rules={[
            () => ({
              transform(value) {
                return Object.values(value).filter((_v) => !isEmpty(_v));
              },
              validator: async (_, valueArr) => {
                const invalidLgcName = valueArr
                  .filter((l) => isEmpty(l.title))
                  .map((x) => HTLanguageSetsMapVal[x.lgc].label)
                  .join(', ');
                if (isNotEmpty(invalidLgcName)) {
                  // Please complete multi -language information
                  return Promise.reject(new Error(`请完善多语种信息: ${invalidLgcName}`));
                }
                return Promise.resolve();
              },
            }),
          ]}>
          <ProductInfoLgc editable={infoEditable} formInstance={form} pickEditedInfo={pickEditedInfo} />
        </Form.Item>

        <Form.Item name='quotation'>
          <ProductInfoQuotation editable={priceEditable} />
        </Form.Item>

        <Form.Item hidden name={'id'} label={'ID'}>
          <Input />
        </Form.Item>
        {/* <Form.Item hidden name={'title'} label={'title'}>
          <Input />
        </Form.Item> */}
        {showSave && (
          <Form.Item>
            <div className='flex justify-around'>
              <Button type='primary' htmlType='submit' loading={loading}>
                {t('common:Save')}
              </Button>
            </div>
          </Form.Item>
        )}
      </Form>
    </>
  );
};

function getFields(props) {
  const { fieldProps, fieldComProps, form, t, dataSets } = props;
  // console.log('getFields', props.initialValue);
  const styleProps = {};
  const editableProps = (name) => {
    // const disabled = props.ignoreEditable ? false : (isEmpty(props.initialValue?.[name]) && props.editable ? false : true)
    const disabled = !props.editable;
    return { disabled, className: disabled ? '!text-slate-500' : '' };
  };
  const bigCol = 4 * 2;
  const midCol = 8;
  const layoutProps = {
    // gutter: { xs: 8, sm: 8, lg: 16 },
    lg: { span: 6 },
    md: { span: 8 },
    sm: { span: 12 },
    xs: { span: 24 },
  };
  const item = (name, sort = 0, render, col) => {
    const customCol = col || midCol;
    const mdCol = customCol * 2;
    return {
      'key': '',
      sort,
      name,
      render,
      'hide': false,
      'col': { lg: { span: customCol }, md: { span: mdCol < 8 ? 10 : mdCol > 24 ? 24 : mdCol }, flex: mdCol < 8 ? '1 0' : '' },
    };
  };
  let baseChildren = [];
  baseChildren = [
    item(
      'product_title',
      99,
      <Form.Item name='product_title' label={t('Title')} {...fieldProps.product_title} rules={[{ required: true }]} tooltip={false}>
        <Input allowClear {...fieldComProps.product_title} {...styleProps} {...editableProps('product_title')} />
      </Form.Item>,
      fieldProps?.product_title?.col || midCol
    ),
    item(
      'code',
      99,
      <Form.Item name='code' label={t('Code')} {...fieldProps.code} rules={[{ required: true }]} tooltip={false}>
        <Input allowClear {...fieldComProps.code} {...styleProps} {...editableProps('code')} />
      </Form.Item>,
      fieldProps?.code?.col || midCol
    ),
    item(
      'city',
      99,
      <Form.Item name='city' label={t('City')} {...fieldProps.city} rules={[{ required: true }]} tooltip={t('FormTooltip.City')}>
        <CitySelector {...styleProps} {...editableProps('city_id')} placeholder={t('FormTooltip.City')} />
      </Form.Item>,
      fieldProps?.city?.col || midCol
    ),
    item(
      'dept',
      99,
      <Form.Item name='dept' label={t('Dept')} {...fieldProps.dept}>
        <DeptSelector labelInValue={false} isLeaf {...styleProps} {...editableProps('dept')} />
      </Form.Item>,
      fieldProps?.dept?.col || midCol
    ),
    item(
      'duration',
      99,
      <Form.Item name='duration' label={t('Duration')} {...fieldProps.duration} rules={[{ required: true, type: 'number', min: 0}]} tooltip={false}>
        <InputNumber suffix={'H'} max={24} {...styleProps} {...editableProps('duration')} />
        {/* <Input allowClear {...fieldComProps.duration} suffix={'H'} /> */}
      </Form.Item>,
      fieldProps?.duration?.col || midCol
    ),
    item(
      'km',
      99,
      <Form.Item name='km' label={t('KM')} {...fieldProps.km} rules={[{ required: true, },]} tooltip={t('FormTooltip.KM')}>
        <InputNumber suffix={'KM'} min={0.1} {...styleProps} {...editableProps('km')} placeholder={t('FormTooltip.KM')} />
      </Form.Item>,
      fieldProps?.km?.col || midCol
    ),
    item(
      'recommends_rate',
      99,
      <Form.Item name='recommends_rate' label={t('RecommendsRate')} {...fieldProps.recommends_rate} tooltip={t('FormTooltip.RecommendsRate')}>
        {/* <Input placeholder={t('RecommendsRate')} allowClear /> */}
        <InputNumber {...styleProps} {...editableProps('recommends_rate')} min={1} max={1000} />
        {/* <Select
          {...styleProps}
          {...editableProps('recommends_rate')}
          style={{ width: '100%' }}
          labelInValue={false}
          options={[
            { value: 1, label: 'Top 1' },
            { value: 2, label: 'Top 2' },
            { value: 3, label: 'Top 3' },
            { value: 4, label: '4' },
            { value: 5, label: '5' },
          ]}
        /> */}
      </Form.Item>,
      fieldProps?.recommends_rate?.col || midCol
    ),
    item(
      'display_to_c',
      99,
      <Form.Item
        name='display_to_c'
        label={t('DisplayToC')}
        {...fieldProps.display_to_c}
        tooltip={t('FormTooltip.DisplayToC')}
        // rules={[
        //   () => ({
        //     validator(_, value) {
        //       if ((value || []).includes(153002) && !(value || []).includes(153001)) {
        //         return Promise.reject(new Error('不允许仅报价信显示'));
        //       }
        //       return Promise.resolve();
        //     },
        //   }),
        // ]}
      >
        {/* <Checkbox.Group
          options={[
            { value: 153001, label: '报价信不显示' },
            { value: 153002, label: '计划不显示' },
          ]}
        /> */}
        <Select
          labelInValue={false}
          options={[
            { value: '153001', label: '在计划显示，不在报价信显示' },
            { value: '0', label: '计划和报价信都要显示' },
            { value: '153001, 153002', label: '计划和报价信都不用显示' },
          ]}
          {...styleProps}
          {...editableProps('display_to_c')}
        />
      </Form.Item>,
      fieldProps?.display_to_c?.col || midCol
    ),
    item(
      'open_weekdays',
      99,
      <Form.Item name='open_weekdays' label={t('OpenWeekdays')} {...fieldProps.open_weekdays} tooltip={false}>
        <Checkbox.Group options={dataSets.weekdays} {...styleProps} {...editableProps('open_weekdays')} />
      </Form.Item>,
      fieldProps?.open_weekdays?.col || 24
    ),
    item(
      'remarks',
      99,
      <Form.Item name='remarks' label={t('Remarks')} {...fieldProps.remarks} tooltip={t('FormTooltip.Remarks')}>
        <Input.TextArea allowClear rows={2} maxLength={2000} showCount {...fieldComProps.remarks} {...styleProps} {...editableProps('remarks')} />
      </Form.Item>,
      fieldProps?.remarks?.col || 24
    ),
  ];
  baseChildren = baseChildren
    .map((x) => {
      x.hide = false;
      if (props.sort === undefined) {
        return x;
      }
      const tmpSort = props.sort;
      for (const key in tmpSort) {
        if (Object.prototype.hasOwnProperty.call(tmpSort, key)) {
          if (x.name === key) {
            x.sort = tmpSort[key];
          }
        }
      }
      return x;
    })
    .map((x) => {
      if (props.hides.length === 0 && props.shows.length === 0) {
        return x;
      }
      if (props.hides.length === 0) {
        x.hide = !props.shows.includes(x.name);
      } else if (props.shows.length === 0) {
        x.hide = props.hides.includes(x.name);
      }
      return x;
    })
    .filter((x) => !x.hide)
    .sort((a, b) => {
      return a.sort < b.sort ? -1 : 1;
    });
  const children = [];
  const leftStyle = {}; // { borderRight: '1px solid #dedede' };
  for (let i = 0; i < baseChildren.length; i++) {
    let style = {}; // { padding: '0px 2px' };
    style = i % 2 === 0 && baseChildren[i].col === 12 ? { ...style, ...leftStyle } : style;
    style = !baseChildren[i].hide ? { ...style, display: 'block' } : { ...style, display: 'none' };
    const Item = (
      <Col key={String(i)} style={style} {...baseChildren[i].col} className='px-1 shrink-0 grow'>
        {baseChildren[i].render}
      </Col>
    );
    children.push(Item);
  }
  return children;
}

const formValuesMapper = (values) => {
  const destinationObject = {
    'city': [
      { key: 'city_id', transform: (value) => value?.value || value?.key || '' },
      { key: 'city_name', transform: (value) => value?.label || '' },
    ],
    'dept': { key: 'dept_id', transform: (value) => (typeof value === 'string' ? value : value?.value || value?.key || '') },
    'open_weekdays': { key: 'open_weekdays', transform: (value) => (Array.isArray(value) ? value.join(',') : value) },
    // 'recommends_rate': { key: 'recommends_rate', transform: (value) => ((typeof value === 'string' || typeof value === 'number') ? value : value?.value || value?.key || '') },
    // 'lgc_details': [
    //   {
    //     key: 'lgc_details',
    //     transform: (value) => {
    //       const _val = value.filter((s) => s !== undefined).map((e) => ({ title: '', ...e, description: e.description || '' }));
    //       return _val || '';
    //     },
    //   },
    //   {
    //     key: 'lgc_details_mapped_tmp',
    //     transform: (value) => {
    //       const _val = value.filter((s) => s !== undefined);
    //       return _val.reduce((r, c) => ({ ...r, [c.lgc]: c }), {});
    //     },
    //   },
    // ],
    'lgc_details_mapped': [
      {
        key: 'lgc_details',
        transform: (value) => {
          const valueArr = Object.values(value)
            .filter((_v) => !isEmpty(_v.lgc))
            .map((e) => ({ title: '', ...e, descriptions: e.descriptions || '' }));
          return valueArr;
        },
      },
      {
        key: 'lgc_details_mapped',
        transform: (value) => {
          const valueArr = Object.values(value)
            .filter((_v) => !isEmpty(_v.lgc))
            .map((e) => ({ title: '', ...e, descriptions: e.descriptions || '' }));
          return valueArr.reduce((r, c) => ({ ...r, [c.lgc]: c }), {});
        },
      },
    ],
    'product_title': { key: 'title' },
  };
  let dest = {};
  const { city, dept, product_title, ...omittedValue } = values;
  dest = { ...omittedValue, ...objectMapper(values, destinationObject) };
  for (const key in dest) {
    if (Object.prototype.hasOwnProperty.call(dest, key)) {
      dest[key] = typeof dest[key] === 'string' ? (dest[key] || '').trim() : dest[key];
    }
  }
  // omit empty
  // Object.keys(dest).forEach((key) => (dest[key] == null || dest[key] === '' || dest[key].length === 0) && delete dest[key]);
  const { lgc_details, lgc_details_mapped, quotation, ...info } = dest;
  return { info, lgc_details, lgc_details_mapped, quotation };
};

export default InfoForm;
