import { useEffect, useState } from 'react';
import { Input, Tabs, Modal, Select, Form } from 'antd';
import useProductsStore from '@/stores/Products/Index';
import { useHTLanguageSets, useHTLanguageSetsMapVal } from '@/hooks/useHTLanguageSets';
import { useTranslation } from 'react-i18next';
import { useDefaultLgc } from '@/i18n/LanguageSwitcher';
import { cloneDeep, isEmpty, isNotEmpty } from '@/utils/commons';

const ProductInfoLgc = ({ editable, formInstance, pickEditedInfo, ...props }) => {
  const { t } = useTranslation('products');
  const { language: languageHT } = useDefaultLgc();
  const HTLanguageSetsMapVal = useHTLanguageSetsMapVal();
  const allLgcOptions = useHTLanguageSets();
  const [editingProduct] = useProductsStore((state) => [state.editingProduct]);
  const [activeKey, setActiveKey] = useState();
  const [items, setItems] = useState([]);

  useEffect(() => {
    formInstance.setFieldValue(['lgc_details_mapped', '2', 'title'], pickEditedInfo.product_title);

    return () => {};
  }, [pickEditedInfo.product_title]);


  useEffect(() => {
    const existsLgc = (editingProduct?.lgc_details || []).map((ele, li) => ({
      ...ele,
      label: HTLanguageSetsMapVal[ele.lgc]?.label || ele.lgc,
      // key: `${editingProduct.info.id}-${ele.id}`,
      key: ele.lgc,
      closable: false, // isPermitted(PERM_PRODUCTS_MANAGEMENT) ? true : false,
      forceRender: true,
      children: (
        <Form.Item noStyle key={`${editingProduct.info.id}-${ele.id}`}>
          <Form.Item name={['lgc_details_mapped', `${ele.lgc}`, 'title']} label={t('products:Title')} initialValue={ele.title} rules={[{ required: true }]} tooltip={t(`FormTooltip.NewTitle.${editingProduct?.info?.product_type_id}`)}>
            <Input
              className={' !text-slate-600'}
              allowClear
              placeholder={t(`FormTooltip.NewTitle.${editingProduct?.info?.product_type_id}`)}
              // onChange={(e) => handleChange('title', e.target.value)}
              // disabled={ignoreEditable ? false : (!isEmpty(ele.title) || !editable)}
              // disabled={ignoreEditable ? false : !editable}
              disabled={!editable}
            />
          </Form.Item>
          <Form.Item name={['lgc_details_mapped', `${ele.lgc}`, 'descriptions']} label={t('products:Description')} initialValue={ele.descriptions} tooltip={t('FormTooltip.Description')}>
            <Input.TextArea
              className={'!text-slate-600'}
              rows={3} maxLength={2000} showCount
              allowClear
              // onChange={(e) => handleChange('description', e.target.value)}
              // disabled={ignoreEditable ? false : (!isEmpty(ele.descriptions) || !editable)}
              // disabled={ignoreEditable ? false : !editable}
              disabled={!editable}
            />
          </Form.Item>
          <Form.Item hidden name={['lgc_details_mapped', `${ele.lgc}`, 'lgc']} initialValue={ele.lgc}>
            <Input />
          </Form.Item>
          <Form.Item hidden name={['lgc_details_mapped', `${ele.lgc}`, 'id']} initialValue={ele.id}>
            <Input />
          </Form.Item>
        </Form.Item>
      ),
    }));
    setItems(existsLgc);
    const pageDefaultLgcI = (editingProduct?.lgc_details || []).findIndex((ele) => ele.lgc === languageHT);
    setActiveKey(existsLgc?.[pageDefaultLgcI || 0]?.key);
    // formInstance.validateFields();
    const filterLgcOptions = allLgcOptions.filter((ele) => !existsLgc.some((item) => `${item.lgc}` === ele.value));
    setLgcOptions(filterLgcOptions);

    return () => {};
  }, [editingProduct]);

  const onLgcTabChange = (newActiveKey) => {
    setActiveKey(newActiveKey);
  };

  const addLgc = (lgcItem) => {
    // const currentLgcOptions = structuredClone(lgcOptions);
    const currentLgcOptions = cloneDeep(lgcOptions);
    currentLgcOptions.splice(
      currentLgcOptions.findIndex((ele) => ele.value === lgcItem.value),
      1
    );
    setLgcOptions(currentLgcOptions);

    const newActiveKey = lgcItem.value;
    const newPanes = [...items];
    newPanes.push({
      ...lgcItem,
      forceRender: true,
      key: lgcItem.value,
      children: (
        <Form.Item noStyle>
          <Form.Item name={['lgc_details_mapped', `${lgcItem.value}`, 'title']} preserve={false} label={t('products:Title')} rules={[{ required: true }]} tooltip={t(`FormTooltip.NewTitle.${editingProduct?.info?.product_type_id}`)}>
            <Input allowClear placeholder={t(`FormTooltip.NewTitle.${editingProduct?.info?.product_type_id}`)} />
          </Form.Item>
          <Form.Item name={['lgc_details_mapped', `${lgcItem.value}`, 'descriptions']} preserve={false} label={t('products:Description')} tooltip={t('FormTooltip.Description')}>
            <Input.TextArea rows={3} maxLength={2000} showCount allowClear />
          </Form.Item>
          <Form.Item hidden name={['lgc_details_mapped', `${lgcItem.value}`, 'lgc']} preserve={false} initialValue={lgcItem.value}>
            <Input />
          </Form.Item>
          <Form.Item hidden name={['lgc_details_mapped', `${lgcItem.value}`, 'id']} preserve={false} initialValue={''}>
            <Input />
          </Form.Item>
        </Form.Item>
      ),
    });
    setItems(newPanes);
    setActiveKey(newActiveKey);
    setSelectNewLgc(null);
  };
  const remove = (targetKey) => {
    let newActiveKey = activeKey;
    let lastIndex = -1;
    items.forEach((item, i) => {
      if (item.key === targetKey) {
        lastIndex = i - 1;
      }
    });
    const newPanes = items.filter((item) => item.key !== targetKey);
    if (newPanes.length && newActiveKey === targetKey) {
      if (lastIndex >= 0) {
        newActiveKey = newPanes[lastIndex].key;
      } else {
        newActiveKey = newPanes[0].key;
      }
    }
    setItems(newPanes);
    setActiveKey(newActiveKey);
    setLgcOptions([...lgcOptions, ...items.filter((item) => item.key === targetKey)]);
  };
  const onEdit = (targetKey, action) => {
    if (action === 'add') {
      setNewLgcModalVisible(true);
    } else {
      remove(targetKey);
      formInstance.validateFields(['lgc_details_mapped']);
    }
  };

  const [newLgcModalVisible, setNewLgcModalVisible] = useState(false);
  const [selectNewLgc, setSelectNewLgc] = useState();
  const [lgcOptions, setLgcOptions] = useState(allLgcOptions);
  const handleOk = () => {
    addLgc(selectNewLgc);
    setNewLgcModalVisible(false);
  };
  const handleCancel = () => {
    setNewLgcModalVisible(false);
  };
  const onSelectNewLgc = (lgcItem) => {
    setSelectNewLgc(lgcItem);
  };
  return (
    <>
      <Tabs
        type='editable-card'
        size='small'
        onChange={onLgcTabChange}
        activeKey={activeKey}
        onEdit={onEdit}
        items={items}
        hideAdd={isEmpty(lgcOptions) || !editable}
        tabPosition='top'
      />
      <Modal title={t('LgcModal.title')} open={newLgcModalVisible} onOk={handleOk} onCancel={() => setNewLgcModalVisible(false)} destroyOnClose>
        <Select
          showSearch
          labelInValue
          style={{ width: '80%' }}
          placeholder={t('LgcModal.placeholder')}
          optionFilterProp='children'
          options={lgcOptions}
          onChange={onSelectNewLgc}
          value={selectNewLgc}
        />
      </Modal>
    </>
  );
};
export default ProductInfoLgc;
