import { useEffect, useState } from 'react';
import { Link, useNavigate } from 'react-router-dom';
import { App, Space, Table, Button, Modal, Divider } from 'antd';
import SearchForm from '@/components/SearchForm';
import dayjs from 'dayjs';
import arraySupport from 'dayjs/plugin/arraySupport';
import { useTranslation } from 'react-i18next';
import useProductsStore, { copyAgencyDataAction } from '@/stores/Products/Index';
import { useProductsTypes, useProductsAuditStatesMapVal } from '@/hooks/useProductsSets';
import useFormStore from '@/stores/Form';
import { objectMapper } from '@/utils/commons';
import CopyProductsFormModal  from './Detail/CopyProducts';
import useAuthStore from '@/stores/Auth';
import RequireAuth from '@/components/RequireAuth';

dayjs.extend(arraySupport);

function Index() {
  const { notification, message } = App.useApp();
  const navigate = useNavigate()
  const { t } = useTranslation();
  const [loading, agencyList, searchAgency] = useProductsStore((state) => [state.loading, state.agencyList, state.searchAgency]);
  const [searchValues, setSearchValues] = useProductsStore((state) => [state.searchValues, state.setSearchValues]);
  const formValuesToSub = useFormStore(state => state.formValuesToSub);

  const stateMapVal = useProductsAuditStatesMapVal();

  const useYear = formValuesToSub.year;

  const handleSearchAgency = (formVal = undefined) => {
    const { starttime, endtime, ...param } = formVal || formValuesToSub;
    const searchParam = objectMapper(param, { agency: 'travel_agency_ids', startdate: 'edit_date1', enddate: 'edit_date2', year: 'use_year' });
    setSearchValues(searchParam);
    searchAgency(searchParam);
  }

  const [copyModalVisible, setCopyModalVisible] = useState(false);
  const [sourceAgency, setSourceAgency] = useState({});
  const [copyAction, setCopyAction] = useState();
  const [copyLoading, setCopyLoading] = useState(false);
  const handleCopyAgency = async (param) => {
    // setCopyLoading(true);
    const toID = param.target_agency;
    // const success = await copyAgencyDataAction({...param, source_agency: sourceAgency.travel_agency_id});
    // setCopyLoading(false);
    // success ? message.success('复制成功') : message.error('复制失败');

    setCopyModalVisible(false);
    navigate(`/products/${toID}/${param.target_use_year || 'all'}/all/edit`);
  };

  const openCopyModal = (from, action) => {
    setSourceAgency(from);
    setCopyAction(action);
    setCopyModalVisible(true);
  };

  useEffect(() => {
    // handleSearchAgency();
  }, []);

  const showTotal = (total) => t('Table.Total', { total });

  const columns = [
    { title: t('products:Vendor'), key: 'vendor', dataIndex: 'travel_agency_name' },
    { title: t('products:CreatedBy'), key: 'poster_by', dataIndex: 'poster_name' },
    { title: t('products:CreateDate'), key: 'poster_date', dataIndex: 'poster_date' },
    {
      title: t('products:AuState'),
      key: 'audit_state',
      dataIndex: 'audit_state',
      render: (_, r) => {
        const stateCls = ` text-${stateMapVal[`${r.audit_state_id}`]?.color} `;
        return <span className={stateCls}>{stateMapVal[`${r.audit_state_id}`]?.label}</span>;
      },
    },
    { title: t('products:AuditedBy'), key: 'audited_by', dataIndex: 'audited_by' },
    { title: t('products:AuditDate'), key: 'audit_date', dataIndex: 'audit_date' },
    {
      title: '',
      key: 'action',
      render: (_, r) => (
        <Space size={'large'}>
          <Link to={`/products/${r.travel_agency_id}/${searchValues.use_year || 'all'}/${searchValues.audit_state || 'all'}/edit`}>{t('Edit')}</Link>
          <Link to={`/products/${r.travel_agency_id}/${searchValues.use_year || 'all'}/${searchValues.audit_state || 'all'}/audit`}>{t('Audit')}</Link>
            {/* {r.audit_state_id >= 0 ? (
            <Link to={`/products/${r.travel_agency_id}/${searchValues.use_year || 'all'}/${searchValues.audit_state || 'all'}/audit`}>{t('Audit')}</Link>
          ) : (
            <span className='text-muted'>{t('Audit')}</span>
          )} */}
          <Button type='link' size={'small'} onClick={() => openCopyModal(r, '#')}>
            {t('Copy') + '-' + t('products:#')}
          </Button>
          {/* <Button type='link' size={'small'} onClick={() => openCopyModal(r, 'o')}>{t('Copy')+'-'+t('products:Offer')}</Button> */}
        </Space>
      ),
    },
  ];
  return (
    <Space direction='vertical' style={{ width: '100%' }}>
      <SearchForm
        fieldsConfig={{
          shows: ['agency', 'audit_state', 'dates', 'year'],
          fieldProps: {
            agency: { col: 4 },
            dates: { label: t('products:CreateDate') },
            keyword: { label: t('products:Title'), col: 4 },
            year: { col: 4, rules: [{ required: true }] },
          },
          sort: { agency: 1, audit_state: 2, keyword: 100 },
        }}
        initialValue={{
          dates: [dayjs().subtract(2, 'M').startOf('M'), dayjs().endOf('M')],
          audit_state: { value: '', label: t('products:State') },
          year: dayjs(), // .add(1, 'year'),
        }}
        onSubmit={(err, formVal, filedsVal) => {
          handleSearchAgency(formVal);
        }}
      />
      <Table bordered={true} columns={columns} dataSource={agencyList} pagination={{ defaultPageSize: 20, showTotal: showTotal }} loading={loading} rowKey={'travel_agency_id'} />

      {/* 复制弹窗 */}
      <CopyProductsFormModal
        initialValues={{agency: {label: sourceAgency.travel_agency_name, value: sourceAgency.travel_agency_id}}}
        open={copyModalVisible}
        action={copyAction}
        source={{ sourceAgency, sourceYear: useYear }}
        onCancel={() => setCopyModalVisible(false)}
        onSubmit={(formVal) => {
          handleCopyAgency(formVal);
        }}
        {...{copyModalVisible, setCopyModalVisible}}
      />
    </Space>
  );
}

export default Index;
